import os
import os.path
import datetime
import subprocess
import sys
import pickle

from gwtimeutils import * 
from variables import *
from gwlogging import logger, gw_stdout, gw_stderr
from GwErrors import GwConfigError


def writeFile(filename, contents):
    try:
        f = open(filename, "w")
        for l in contents:
            f.write("%s\n" % l)
        f.close()
    except Exception as e:
        logger.error("Error occured on writing %s" % filename)
        print type(e)
        print e.args


def packFiles(mode):
        """pack the files into a tarball ready for sending"""
        now = datetime.datetime.utcnow()

        # Rename data files to be unique, prepend date.
        logger.debug("Renaming data files to be unique")
        for fname in os.listdir(BASE_DIR + DATA_DIR):
                if os.path.isfile("%s/%s" % (BASE_DIR + DATA_DIR, fname)):
                        os.rename("%s/%s" % (BASE_DIR + DATA_DIR, fname),
                        "%s/%i-%2.2i-%2.2i-%2.2i.%2.2i-%s" % (
                                BASE_DIR + DATA_DIR,
                                now.year, now.month, now.day, now.hour,
                                now.minute, fname))

        # Create the new tar filename:
        tarf = "%s_data_%i-%2.2i-%2.2i-%2.2i.%2.2i.tar.gz" % (mode, now.year,
                        now.month, now.day, now.hour, now.minute)

        # Tar up stuff
        p = subprocess.Popen(args="/bin/tar -czf ./%s/%s ./%s" % (QUEUED_DIR,
                                tarf, DATA_DIR),
                                cwd=BASE_DIR, shell=True, stdout=gw_stdout,
                                stderr=gw_stderr)
        p.wait()

        # Delete the tarred files
        for fname in os.listdir(BASE_DIR + DATA_DIR):
                #if os.path.isfile( fname ):
                if os.path.isfile("%s/%s" % (BASE_DIR + DATA_DIR, fname)):
                        os.remove("%s/%s" % (BASE_DIR + DATA_DIR, fname))

        logger.info("Tar created and tarred files deleted.")


def wait():
        return os.path.exists(WAIT_FILE)


def stay_awake(start_time):
    """Returns True if we need to stay awake."""
    # Always stay awake if this file exists
    if os.path.exists(STAY_AWAKE_FILE):
        return True
    # Always stay awake for some period of time
    if (datetime.datetime.utcnow() - start_time) < MIN_AWAKE_TIME:
        return True
    else:
        return False


# Execute special file
def executeSpecial():
        if os.path.exists(SPECIAL_FILE):
                logger.info("Executing special file")
                special = subprocess.Popen(args="sh %s" % SPECIAL_FILE,
                                                cwd=(BASE_DIR + DATA_DIR),
                                                shell=True,
                                                stdout=gw_stdout,
                                                stderr=gw_stderr)
                special.wait()


def file_size(filename):
        """Returns the file size of the specified file"""
        return os.path.getsize(filename)


def increment_boot_count():
    try:
        f = open(BOOT_FILE, "r")
        for l in f:
            l = l.strip()
            if len(l) > 0:
                boot_count = int(l)
                break
        f.close()
    except IOError:
        logger.error("Boot file not read - assuming first boot")
        boot_count = 0  # will be incremented later
    if boot_count > 1600:
        logger.error("Boot count:%d" % boot_count)
        logger.error("Replace soon")
    elif boot_count > 1000:
        logger.warn("Boot count:%d" % boot_count)
    else:
        logger.info("Boot count %d" % boot_count)
    boot_count += 1
    f = open(BOOT_FILE, "w")
    f.write(str(boot_count) + "\n")
    f.close()


def get_last_run_time():
    """Reads the last time of execution from a file
    Returns a datetime object"""
    try:
        f = open(INFO_FILE, "r")
        t = None
        for l in f:
            l = l.strip()
            if len(l) > 0:
                t = datetime.datetime.utcfromtimestamp(int(l))
                break
        f.close()
        return t
    except IOError, e:
        logger.error("Info file not read")
        return None


def set_last_run_time():
    ifile = open(INFO_FILE, "w")
    ifile.write(str(totimestamp(datetime.datetime.utcnow())) + "\n")
    ifile.close()


def save_config(conf, filename):
    f = open(filename, "w")
    logger.debug("%s opened for writing" % filename)
    pickle.dump(conf, f)
    logger.info("Config saved to %s" % filename)
    f.close()


def load_config(filename):
    try:
        f = open(filename, "r")
        logger.debug("%s opened for reading" % filename)
        c = pickle.load(f)
        logger.info("Config loaded from %s" % filename)
        f.close()
    except (pickle.UnpicklingError, IOError):
        raise GwConfigError
    return c
