# gwutil.py - a few utility functions

import datetime
import time
import subprocess
import sys
import os
import signal
import os.path
from math import log


from variables import *
from gwlogging import gw_stdout, gw_stderr

import iceCape

from gwfetchutils import *
from gwtimeutils import *
from gwfileutils import *
from gwtimeutils import *
from gwfileutils import *


def mask2number(mask):
        """Returns the number of a sensor ID as opposed to the mask
                for example 128 -> 7
        """
        return int(log(mask, 2))


def fs_sync():
    # Sync the filesystem before we power off...
    p = subprocess.Popen(args="/bin/sync",
                            shell=True,
                            stdout=gw_stdout,
                            stderr=gw_stderr)
    p.wait()


def poweroff(gum, testing):
    if not testing:
        gum.power_off_gumstix(15)
        fs_sync()
        p = subprocess.Popen(args="/sbin/poweroff",
            shell=True,
            stdout=gw_stdout,
            stderr=gw_stderr)
        p.wait()
    else:
        logger.warn("Testing not powering off")
        sys.exit(0)


def int2bin(n, count=16):
    """returns the binary of integer n, using count number of digits"""
    #http://www.daniweb.com/software-development/python/code/216539/
    #decimal-to-binary-conversion-python
    return "".join([str((n >> y) & 1) for y in range(count - 1, -1, -1)])


def format_status(b):
    s = "|"
    for c in b:
        if c == "1":
            s += " On  |"
        else:
            s += " Off |"
    return s


def recover_time(gum, last_run_time):
    logger.debug("Seeing if MSP time is ok")
    dt = gum.get_datetime()
    if dt > last_run_time:
        logger.info("Time on MSP ok")
        set_system_time(dt)
        return True
    else:
        logger.error("Assuming the MSP lost power, resetting" +
               " to state 0 for safety.")
        gum.turn_on_sensor_output(iceCape.GPS)
        rc = gps.set_system_time()
        gum.turn_off_sensor_output(iceCape.GPS)
        if rc == 0:
            #success
            gum.setdatetime()
            return True
        else:
            #failure:
            return False
